"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _combinedBuf = _interopRequireDefault(require("../buf/combined-buf"));

var _nodeBuf = _interopRequireDefault(require("./node-buf"));

var _error = require("../../error");

var _buffer = _interopRequireDefault(require("buffer"));

var _string_decoder = require("string_decoder");

/**
 * Copyright (c) 2002-2020 "Neo4j,"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var decoder = new _string_decoder.StringDecoder('utf8');

function encode(str) {
  return new _nodeBuf["default"](newNodeJSBuffer(str));
}

function decode(buffer, length) {
  if (buffer instanceof _nodeBuf["default"]) {
    return decodeNodeBuffer(buffer, length);
  } else if (buffer instanceof _combinedBuf["default"]) {
    return decodeCombinedBuffer(buffer, length);
  } else {
    throw (0, _error.newError)("Don't know how to decode strings from '".concat(buffer, "'"));
  }
}

function decodeNodeBuffer(buffer, length) {
  var start = buffer.position;
  var end = start + length;
  buffer.position = Math.min(end, buffer.length);
  return buffer._buffer.toString('utf8', start, end);
}

function decodeCombinedBuffer(buffer, length) {
  return streamDecodeCombinedBuffer(buffer, length, function (partBuffer) {
    return decoder.write(partBuffer._buffer);
  }, function () {
    return decoder.end();
  });
}

function streamDecodeCombinedBuffer(combinedBuffers, length, decodeFn, endFn) {
  var remainingBytesToRead = length;
  var position = combinedBuffers.position;

  combinedBuffers._updatePos(Math.min(length, combinedBuffers.length - position)); // Reduce CombinedBuffers to a decoded string


  var out = combinedBuffers._buffers.reduce(function (last, partBuffer) {
    if (remainingBytesToRead <= 0) {
      return last;
    } else if (position >= partBuffer.length) {
      position -= partBuffer.length;
      return '';
    } else {
      partBuffer._updatePos(position - partBuffer.position);

      var bytesToRead = Math.min(partBuffer.length - position, remainingBytesToRead);
      var lastSlice = partBuffer.readSlice(bytesToRead);

      partBuffer._updatePos(bytesToRead);

      remainingBytesToRead = Math.max(remainingBytesToRead - lastSlice.length, 0);
      position = 0;
      return last + decodeFn(lastSlice);
    }
  }, '');

  return out + endFn();
}

function newNodeJSBuffer(str) {
  // use static factory function present in newer NodeJS versions to create a buffer containing the given string
  // or fallback to the old, potentially deprecated constructor
  if (typeof _buffer["default"].Buffer.from === 'function') {
    return _buffer["default"].Buffer.from(str, 'utf8');
  } else {
    // eslint-disable-next-line node/no-deprecated-api
    return new _buffer["default"].Buffer(str, 'utf8');
  }
}

var _default = {
  encode: encode,
  decode: decode
};
exports["default"] = _default;